"""
This is a comment.
"""
#from LogManagerInterfaces import LogManagerInterface
#from LogManagerInterfaces import LogManagerRuleInterface
#from LogManagerInterfaces import LogManagerTargetInterface
import string, traceback
import __main__

PSUBJECTS = ['subsystem', 'severity', 'summary', 'detail', 'error']
PREDICATE_TYPES = ['callable', 're', 'expr', 'N/A']

class LogManagerException:
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value

class LogManagerRuleException(LogManagerException): pass

class LogManager:
    """
    Log Manager class -- accepts log messages and uses
    rules processor to determine to which loggers to send a message
    """

    # __implements__ = LogManagerInterface
    
    def __init__(self, targets={}):
        self.ruler = LogRulesProcessor()
        self.on = 1
        if not __main__.__builtins__.__dict__.get('_log_targets', None):
            __main__.__builtins__._log_targets = targets
        
    def isActivated(self):
        return self.on
    
    def activate(self):
        self.on = 1

    def deactivate(self):
        self.on = 0

    def getRegisteredTargetNames(self):
        return __main__.__builtins__._log_targets.keys()

    def addRegisteredTarget(self, name, target):
        if __main__.__builtins__._log_targets.has_key(name):
            raise LogManagerException,('Target %s exists.' % name)
        __main__.__builtins__._log_targets.update({name:target})
        
    def removeRegisteredTarget(self, name):
        del __main__.__builtins__._log_targets[name]
                
    def getRuleTargets(self):
        return self.ruler.getRuleTargets()
                    
    def appendRule(self, rule):
        self.ruler.appendRule(rule)

    def popRule(self, index):
        return self.ruler.popRule(index)

    def insertRule(self, index, rule):
        self.ruler.insertRule(index, rule)

    def getRule(self, index):
        return self.ruler.getRule(index)

    def getRuleByName(self, name):
        return self.ruler.getRuleByName(name)

    def getRules(self):
        return self.ruler.getRules()
        
    def matchRules(self, tup):
        return self.ruler.matchRules(tup)

    def LOG(self, subsystem, severity, summary, detail='', error=None,
            reraise=None):
        """ accept a log message for input """
        registered_targets = __main__.__builtins__._log_targets
        tup = (subsystem, severity, summary, detail, error, reraise)
        target_names = self.matchRules(tup)
        if self.isActivated():
            for target_name in target_names:
                try:
                    # this is in case rules have stale targets
                    # i'm not sure how else to handle this in a
                    # ZEO'ed environment
                    target = registered_targets.get(target_name, None)
                    if target:
                        apply(target, tup[:-1])
                except:
                    print ("Misconfigured log target %s or "
                           "bad log message %s" %(target_name, tup))
                    traceback.print_exc()
        if reraise:
            raise

class LogRulesProcessor:
    """ Rules processor class for deciding which loggers get a message """
    
    def __init__(self):
        self.rules = []

    def matchRules(self, tup):
        """ accept a tuple as an input and match against rule list """
        destinations = []
        
        for x in self.rules:
            if x.isActivated():
                if x.match(tup, destinations) == -1: break # stop rule
                
        return destinations
                
    def appendRule(self, rule):
        self.rules.append(rule)
        
    def popRule(self, index):
        return self.rules.pop(index)
        
    def insertRule(self, index, rule):
        self.rules.insert(index, rule)
        
    def clearRules(self):
        self.rules = []

    def getRules(self):
        return tuple(self.rules)

    def getRule(self, index):
        return self.rules[index]

    def getRuleByName(self, name):
        for rule in self.rules:
            if rule.getName() == name: return rule
        
    def getRuleTargets(self):
        l = []
        for rule in self.rules:
            targets = rule.getRuleTargets()
            for x in targets:
                if x not in l:
                    l.append(x)
        return l

class FakeLogTarget:
    """ Fake log target, used for testing """

    #__implements__ = LogManagerTargetInterface
    
    def __init__(self):
        self.history = []

    def __call__(self, subsystem, severity, summary, detail, error):
        tup = (subsystem, severity, summary, detail, error)
        self.history.append(tup)


class BaseRule:
    """ Abstract base class for all rules """
    def isActivated(self):
        """ """
        return self.on

    def activate(self):
        self.on = 1

    def deactivate(self):
        self.on = 0

    def getRuleTargets(self):
        """ """
        return list(self.target_names)

    def addRuleTarget(self, name):
        """ """
        self.target_names.append(name)

    def removeRuleTarget(self, name):
        """ """
        self.target_names.remove(target)
                
    def getName(self):
        """ """
        return self.name

    def getRuleType(self):
        """ """
        return self.rule_type

    def getSubject(self):
        """ """
        return self.subject

    def getPredicateType(self):
        """ """
        return self.predicate_type

    def getPredicateAsString(self):
        """ """
        pt = self.predicate_type
        if pt == 'expr':
            return self.predicate
        elif pt == 're':
            return self.predicate.pattern
        elif pt == 'callable':
            if hasattr(self.predicate, 'id'):
                return self.predicate.id
            else:
                return self.predicate.__name__
        elif pt == 'N/A':
            return pt

class ConditionalRule(BaseRule):
    """ Abstract base class for conditional rules """
    def __init__(self, name, subject, predicate, target_names):
        self.name = name

        self.on = 1

        if type(target_names) is type([]):
            self.target_names = target_names
        else:
            err = ("Targets names must be provided as a list (e.g. " 
                   "['target1','target2']. Instead, passed %s." %
                   target_names)
            raise LogManagerRuleException, err
        
        if type(subject) is type(''):
            subject = string.strip(string.lower(subject))
            
            if subject in PSUBJECTS:
                self.subject = subject
            else:
                err = "Subject must be one of %s." % PSUBJECTS
                raise LogManagerRuleException, err
            
        else:
            err = "Subject %s must be a string." % subject
            raise LogManagerRuleException, err

        predicate_type = _getPredicateType(predicate)
        if predicate_type in PREDICATE_TYPES:
            self.predicate = predicate
            self.predicate_type = predicate_type
        else:
            err = ("Predicate must be a string which contains a Python "
                   "expression, a callable object, or a compiled regular "
                   "expression.  Instead, received %s." % predicate)
            raise LogManagerRuleException, err

    def _getMatchResult(self, tup):
        pt = self.predicate_type
        if pt == 'expr':
            try:
                return eval(self._getExpression(tup),
                            {'__builtins__': {'None':None}})
            except:
                print "Bad simple expr in log rule '%s'." % self.getName()
                print tup
        else:
            val = self._getSubjectValue(tup)
            if pt =='re':
                return self.predicate.search(val)
            if pt == 'callable':
                return self.predicate(val)
            if pt == None:
                return 'N/A'
    
    def _getExpression(self, tup):
        val = self._getSubjectValue(tup)
        if type(val) is type(''):
            val = r"""'%s'""" % val
        return "%s %s" % (val, self.predicate)

    def _getSubjectValue(self, tup, lower=string.lower, strip=string.strip):
        index = PSUBJECTS.index(strip(lower(self.subject)))
        return tup[index]

class UnconditionalRule(BaseRule):
    """ Abstract base class for unconditional rules """
    def __init__(self, name, subject, predicate, target_names):
        self.name = name
        self.subject = "N/A"
        self.predicate = "N/A"
        self.predicate_type = "N/A"
        self.on = 1

        if type(target_names) is type([]):
            self.target_names = target_names
        else:
            err = ("Targets names must be provided as a list (e.g. " 
                   "['target1','target2']. Instead, passed %s." %
                   target_names)
            raise LogManagerRuleException, err

# concrete rule classes follow

class StopRule(ConditionalRule):
    """ Concrete stop rule class """

    #__implements__ = LogManagerRuleInterface

    rule_type = 'stop'

    def match(self, tup, destinations):
        res = self._getMatchResult(tup)
        if res:
            return -1
        else:
            return 0

class UnconditionalLogRule(UnconditionalRule):
    """ Concrete unconditional log rule class """

    #__implements__ = LogManagerRuleInterface

    rule_type = 'unconditional log'

    def match(self, tup, destinations):
        dappend = destinations.append
        for target in self.target_names:
            if target not in destinations:
                dappend(target)
        return 1
    
class ConditionalLogRule(ConditionalRule):
    """ Concrete conditional log rule class """

    #__implements__ = LogManagerRuleInterface

    rule_type = 'conditional log'

    def match(self, tup, destinations):
        res = self._getMatchResult(tup)
        if res:
            dappend = destinations.append
            for target in self.target_names:
                if target not in destinations:
                    dappend(target)
            return 1
        else:
            return 0

class UnconditionalSuppressRule(UnconditionalRule):
    """ Concrete unconditional suppress rule class """

    #__implements__ = LogManagerRuleInterface

    rule_type = 'unconditional suppress'

    def __init__(self, name, subject, predicate, target_names):
        self.name = name
        self.subject = ''
        self.predicate = ''
        self.predicate_type = ''
        self.on = 1

        if type(target_names) is type([]):
            self.target_names = target_names
        else:
            err = ("Targets names must be provided as a list (e.g. " 
                   "['target1','target2']. Instead, passed %s." %
                   target_names)
            raise LogManagerRuleException, err
        

    def match(self, tup, destinations):
        dremove = destinations.remove
        for target in self.target_names:
            if target in destinations:
                dremove(target)
        return 1

class ConditionalSuppressRule(ConditionalRule):
    """ Concrete conditional suppress rule class """

    #__implements__ = LogManagerRuleInterface

    rule_type = 'conditional suppress'

    def match(self, tup, destinations):
        res = self._getMatchResult(tup)
        if res:
            dremove = destinations.remove
            for target in self.target_names:
                if target in destinations:
                    dremove(target)
            return 1
        else:
            return 0


# Utility functions

def _getPredicateType(predicate):
    if type(predicate) == type(''):
        return 'expr'
    if (hasattr(predicate, '__class__') and
        hasattr(predicate.__class__, '__name__') and
        predicate.__class__.__name__ == 'RegexObject'):
        return 're'
    if callable(predicate):
        return 'callable'
    if predicate is None:
        return 'N/A'
    # the bit about asking the predicate if it's a RegexObject
    # should be replaced with an interface assertion when possible.
    # this will probably break with sre.
