try:
    import Interface
except:
    import os, sys
    program = sys.argv[0]
    here = os.path.join(os.getcwd(), os.path.split(program)[0])
    sys.path.insert(0, '../..')
    import Interface

class EventLogManagerInterface(Interface.Base):
    """
    Interface for the Event Log Manager, a Zope wrapper for the Log Manager.
    """

    def __init__(self, id, title):
        """
        Initializes the Event Log Manager with default targets and a
        default rule.  The Event Log Manager becomes a wrapper for a
        Log Manager instance.  Targets are decided during the initialization
        of the Event Log Manager via a configuration file (see the
        README).
        """

    def isActivated(self):
        """
        Returns 1 if Event Log Manager is activated, 0 if Event Log Manager is
        deactivated.
        """
    def activate(self, REQUEST):
        """
        Activates the Event Log Manager.  Redirects to management interface
        if REQUEST is not None.
        """

    def deactivate(self, REQUEST):
        """
        Deactivates the LogManager. Deactivating the LogManager stops
        all logging.  Redirects to management interface if REQUEST is not
        None.
        """
        
    def addRule(self, rule_type, rule_name, target_names=[], REQUEST=None):
        """
        Adds a rule of rule_type with rule_name and targets == target_names.
        Redirects to management interface if REQUEST is not None.  rule_type
        is one of 'conditional log', 'conditional suppress', 'unconditional
        log', unconditional suppress', or 'stop'.  rule_name is a string.
        target_names is a sequence of strings.
        """

    def removeRule(self, index, REQUEST=None):
        """
        Removes the rule numbered 'index' (an integer).  Redirects to
        management interface if REQUEST is not None.
        """

    def LOG(self, subsystem, severity, summary, detail='', error=None,
            reraise=None):
        """
        Logs a message.
        """
        
    __call__=LOG


class LogManagerInterface(Interface.Base):
    """
    Interface for the Log Manager, a component designed to
    handle traffic direction for log messages.
    """

    def __init__(self, targets={}):
        """
        Initializes the Log Manager.  Targets is a dictionary in the form
        {'target_name':callableobject}.  This dictionary is put into
        the __main__ module's __builtins__ namespace as '_log_targets' unless
        an attribute of that name already exists within that namespace.
        """

    def isActivated(self):
        """
        Returns 1 if LogManager is activated, 0 if LogManager is
        deactivated.
        """
    def activate(self):
        """
        Activates the LogManager.
        """

    def deactivate(self):
        """
        Deactivates the LogManager. Deactivating the LogManager stops
        all logging.
        """

    def getRegisteredTargetNames(self):
        """
        Returns the list of registered Target object names.
        """

    def addRegisteredTarget(self, name, target):
        """
        Adds a registered Target.  'name' is a string.  'target' must be an
        object that implements the LogManagerTargetInterface.
        """

    def removeRegisteredTarget(self, name):
        """
        Removes a registered Target.  'name' refers to the name
        of the registered Target to remove.
        """

    def appendRule(self, rule):
        """
        Takes an object which implements the LogManagerRuleInterface object
        and appends it to the list of rules.
        """

    def popRule(self, index):
        """
        Removes the rule number specified by 'index'.
        """

    def insertRule(self, index, rule):
        """
        Inserts 'rule' in the rule list at the 'index' specified.
        """

    def getRule(self, index):
        """
        Gets the rule object at 'index'.
        """

    def getRuleByName(self, name):
        """
        Gets the rule obect named 'name'.
        """

    def getRules(self):
        """
        Returns all of the ordered rule objects living within this
        Log Manager in a Python list.
        """ 
        
    def LOG(self, subsystem, severity, summary, detail='', error=None,
            reraise=None):
        """
        Accept a log message for processing.  Rules are applied to
        the incoming message in a predefined order, and targets are sent
        the provided message as decided by the rule processor.
        """

class LogManagerRuleInterface(Interface.Base):
    """
    Interface for LogManagerRules, which are objects that are iterated
    over during the logging of each message.  Rules decide how and
    if a message is logged.
    """
    def __init__(self, name, subject, predicate, target_names):
        """
        Intitialization routine for rule.  'name' is a string identifying
        the rule.  'subject' is the subject of the message on which to
        operate (generally one of ['subsystem','severity','summary',
        'detail', 'error'].  'predicate' is one of: a) the string
        value of a Python expression predicate (e.g. "==ZODB"), b) a
        callable object (e.g. a Python method), or c) a compiled
        regular expression object.  'target_names' are the target names
        on which the rule should operate in a Python list.
        """

    def isActivated(self):
        """
        Returns 1 if LogManager is activated, 0 if LogManager is
        deactivated.
        """

    def activate(self):
        """
        Activates the LogManager.
        """

    def deactivate(self):
        """
        Deactivates the LogManager.
        """

    def getRuleTargets(self):
        """
        Returns the names of targets assigned to this rule in a Python list.
        """

    def addRuleTarget(self, name):
        """
        Adds a target name to the rule.
        """

    def removeRuleTarget(self, name):
        """
        Removes the rule target named 'name'. 
        """

    def getName(self):
        """
        Returns the name of the rule as a string.
        """

    def getRuleType(self):
        """
        Returns the "rule type", one of 'stop', 'unconditional log',
        'conditional log', 'uncondititional suppress', or 'conditional
        suppress'.
        """

    def getSubject(self):
        """
        Returns the subject of the rule, one of ['subsystem', 'severity',
        'summary', 'detail', 'error'].
        """

    def getPredicateType(self):
        """
        Returns one of ['re', 'expr', 'callable'] depending on predicate
        type.
        """

    def getPredicateAsString(self):
        """
        Returns the rule's predicate in a representable format.
        """

    def match(self, tup, destinations):
        """
        Accepts (tup, destinations) where 'tup' is a tuple containing the
        standard log message fields (subsystem, severity, summary, detail,
        error, reraise), and 'destinations' is Python list which gets mutated
        during rules processing.  'destinations' starts out empty.  As rules
        are processed, target names are appended to it or removed from it as
        decided by rule processing.
        
        If this rule is a 'Log Rule', the rule's targets are added to the
        destinations list.  If this rule is a 'Suppress Rule', the rule's
        targets are removed from the destination list.  If this rule is a
        'Stop Rule', any further processing of the message is stopped.
        'Log Rules' and 'Suppress Rules' may be either conditional
        or unconditional.  Conditional rules use a subject/predicate
        processing to determine whether the rule matches the message, after
        which the destination list is acted upon. Unconditional rules do no
        subject/predicate processing, and always match, therefore
        the destination list is always acted upon.  Stop rules are always
        conditional.

        The five derived rule types, as described above, are:

        unconditonal log
        conditional log
        unconditional suppress
        conditional suppress
        stop
        """

class LogManagerTargetInterface(Interface.Base):
    """
    Log Manager Target interface.  A Log Manager Target is an object
    which actually does the logging of messages (such as a FileTarget or
    a SyslogTarget).  These kinds of objects are used by the Log Manager
    to log a message after rules processing has occurred.
    """
    def __call__(self, subsystem, severity, summary, detail='', error=None):
        """
        Logs a log message.
        """

